"""
TrajAI Examples Package

This package contains practical, runnable examples demonstrating how to use TrajAI
to test agent behavior. Each scenario is self-contained and can be run independently.

See README.md for an overview of all scenarios.
"""
